SystemOrganization addCategory: #'Seaside-FileSystem'!

WARequestHandler subclass: #WAExternalFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-FileSystem'!

!WAExternalFileLibrary class methodsFor: 'accessing' stamp: 'lr 2/10/2008 14:49'!
description
	^ 'File Directory'! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'jf 2/15/2010 02:13'!
asAbsoluteUrl: aFileName relativeTo: aRequest
	^aRequest url withoutQuery
		addToPath: aFileName;
		yourself! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 4/20/2008 21:18'!
contentsOfFile: aFilename
	| stream |
	stream := aFilename readStream.
	^[
		stream binary.
		stream upToEnd ]
		ensure: [ stream close ]! !

!WAExternalFileLibrary methodsFor: 'configuration' stamp: 'pmm 10/18/2008 10:56'!
defaultConfiguration
	^ super defaultConfiguration
		addParent: WAExternalFileLibraryConfiguration instance;
		yourself! !

!WAExternalFileLibrary methodsFor: 'accessing' stamp: 'pmm 10/18/2008 11:10'!
directory
	| path filename |
	path := self preferenceAt: #directory.
	filename := SpFilename named: path.
	"hack around SPort bugs"
	^(path last = filename separator)
		ifTrue: [ SpFilename named: path allButLast ]
		ifFalse: [ filename ]! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 8/31/2009 14:21'!
filenameForConsumer: aPathConsumer
	"Answer the SpFilename pointing to the file for the given request."
	| filename |
	filename := self directory.
	[ aPathConsumer atEnd ] whileFalse: [
		filename := filename construct: aPathConsumer next ].
	^filename! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 11/1/2010 22:14'!
filesIn: aPath do: aOneArgumentBlock 
	"this can not be done with SPort"
	(GRPlatform current filesIn: aPath) do: [ :each |
		aOneArgumentBlock value: (GRPlatform current localNameOf: each) ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'pmm 9/5/2009 10:49'!
handleFiltered: aRequestContext
	[ self processContext: aRequestContext ] 
		on: Error
		do: [ :error | 
			aRequestContext responseGenerator
				internalError: error;
				respond ]! !

!WAExternalFileLibrary methodsFor: 'accessing' stamp: 'pmm 10/18/2008 11:10'!
listing
	^ self preferenceAt: #listing! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:30'!
mimetypeForRequest: aRequest
	| path |
	path := aRequest url path.
	^(path isEmpty or: [ (path last includes: $.) not ])
		ifTrue: [ WAFileLibrary defaultMimeType seasideMimeType ]
		ifFalse: [
			| extension |
			extension := path last copyAfterLast: $..
			WAFileLibrary mimetypeFor: extension ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'pmm 8/31/2009 14:22'!
processContext: aRequestContext
	| filename |
	filename := self filenameForConsumer: aRequestContext consumer.
	(self listing and: [ filename isDirectory ])
		ifTrue: [ self
				processDirectory: filename asString "#asString on SPFilename"
				context: aRequestContext ]
		ifFalse: [ self processFile: filename context: aRequestContext ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'pmm 11/1/2010 22:14'!
processDirectory: aString context: aRequestContext 
	| contents |
	contents := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :html | html title: aRequestContext request url greaseString ];
		render: [ :html | 
			html heading
				level: 1;
				with: aRequestContext request url pathString.
			html unorderedList:  [
				self 
					filesIn: aString
					do: [ :each | 
						html listItem: [
							html anchor
								url: (self 
									asAbsoluteUrl: each
									relativeTo: aRequestContext request);
								with: each ] ] ] ].
	aRequestContext respond: [ :response | 
		response
			contentType: WAMimeType textHtml;
			nextPutAll: contents ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'pmm 10/28/2010 18:11'!
processFile: aFilename context: aRequestContext
	aRequestContext respond: [ :response |
		response
			contentType: (self mimetypeForRequest: aRequestContext request);
			binary;
			nextPutAll: (self contentsOfFile: aFilename) ]! !

WASystemConfiguration subclass: #WAExternalFileLibraryConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-FileSystem'!

!WAExternalFileLibraryConfiguration methodsFor: 'description' stamp: 'pmm 2/20/2009 11:59'!
describeOn: config
	(config string: #directory)
		label: 'Directory';
		comment: 'The base directory to serve files from.';
		default: (SpFilename named: 'xxxDoesNotExists') asAbsoluteFilename directory asString. "#asString on SPFilename"
	(config boolean: #listing)
		label: 'Allow Listing';
		comment: 'Enable listing of directory entries.';
		default: false! !

!GRPackage class methodsFor: '*seaside-filesystem' stamp: 'lr 2/10/2010 11:28'!
seasideFileSystem
	^ self new
		name: 'Seaside-FileSystem';
		description: 'File library that serves files from external directory.';
		addDependency: 'Seaside-Core';
		url: #seasideLGPLUrl;
		license: #LGPL;
		yourself! !
